@@ -1,6 +1,7 @@
 import os
 import sys
-from copy import copy
+import warnings
+from copy import copy, deepcopy
 from collections import deque
 from contextlib import contextmanager
 from enum import Enum
@@ -10,7 +11,9 @@ from itertools import chain
 
 from sentry_sdk.attachments import Attachment
 from sentry_sdk.consts import DEFAULT_MAX_BREADCRUMBS, FALSE_VALUES, INSTRUMENTER
-from sentry_sdk.profiler import Profile
+from sentry_sdk.feature_flags import FlagBuffer, DEFAULT_FLAG_CAPACITY
+from sentry_sdk.profiler.continuous_profiler import try_autostart_continuous_profiler
+from sentry_sdk.profiler.transaction_profiler import Profile
 from sentry_sdk.session import Session
 from sentry_sdk.tracing_utils import (
     Baggage,
@@ -25,15 +28,19 @@ from sentry_sdk.tracing import (
     Span,
     Transaction,
 )
-from sentry_sdk._types import TYPE_CHECKING
 from sentry_sdk.utils import (
+    capture_internal_exception,
     capture_internal_exceptions,
     ContextVar,
+    datetime_from_isoformat,
+    disable_capture_event,
     event_from_exception,
     exc_info_from_error,
     logger,
 )
 
+from typing import TYPE_CHECKING
+
 if TYPE_CHECKING:
     from collections.abc import Mapping, MutableMapping
 
@@ -150,7 +157,7 @@ def _disable_capture(fn):
     return wrapper  # type: ignore
 
 
-class Scope(object):
+class Scope:
     """The scope holds extra information that should be sent with all
     events that belong to it.
     """
@@ -185,6 +192,8 @@ class Scope(object):
         "_propagation_context",
         "client",
         "_type",
+        "_last_event_id",
+        "_flags",
     )
 
     def __init__(self, ty=None, client=None):
@@ -216,6 +225,7 @@ class Scope(object):
         rv = object.__new__(self.__class__)  # type: Scope
 
         rv._type = self._type
+        rv.client = self.client
         rv._level = self._level
         rv._name = self._name
         rv._fingerprint = self._fingerprint
@@ -240,6 +250,10 @@ class Scope(object):
 
         rv._profile = self._profile
 
+        rv._last_event_id = self._last_event_id
+
+        rv._flags = deepcopy(self._flags)
+
         return rv
 
     @classmethod
@@ -308,6 +322,23 @@ class Scope(object):
 
         return _global_scope
 
+    @classmethod
+    def last_event_id(cls):
+        # type: () -> Optional[str]
+        """
+        .. versionadded:: 2.2.0
+
+        Returns event ID of the event most recently captured by the isolation scope, or None if no event
+        has been captured. We do not consider events that are dropped, e.g. by a before_send hook.
+        Transactions also are not considered events in this context.
+
+        The event corresponding to the returned event ID is NOT guaranteed to actually be sent to Sentry;
+        whether the event is sent depends on the transport. The event could be sent later or not at all.
+        Even a sent event could fail to arrive in Sentry due to network issues, exhausted quotas, or
+        various other reasons.
+        """
+        return cls.get_isolation_scope()._last_event_id
+
     def _merge_scopes(self, additional_scope=None, additional_scope_kwargs=None):
         # type: (Optional[Scope], Optional[Dict[str, Any]]) -> Scope
         """
@@ -476,7 +507,7 @@ class Scope(object):
         Returns the Sentry "sentry-trace" header (aka the traceparent) from the
         currently active span or the scopes Propagation Context.
         """
-        client = Scope.get_client()
+        client = self.get_client()
 
         # If we have an active span, return traceparent from there
         if has_tracing_enabled(client.options) and self.span is not None:
@@ -491,7 +522,7 @@ class Scope(object):
             return traceparent
 
         # Fall back to isolation scope's traceparent. It always has one
-        return Scope.get_isolation_scope().get_traceparent()
+        return self.get_isolation_scope().get_traceparent()
 
     def get_baggage(self, *args, **kwargs):
         # type: (Any, Any) -> Optional[Baggage]
@@ -499,7 +530,7 @@ class Scope(object):
         Returns the Sentry "baggage" header containing trace information from the
         currently active span or the scopes Propagation Context.
         """
-        client = Scope.get_client()
+        client = self.get_client()
 
         # If we have an active span, return baggage from there
         if has_tracing_enabled(client.options) and self.span is not None:
@@ -516,7 +547,7 @@ class Scope(object):
                 return Baggage(dynamic_sampling_context)
 
         # Fall back to isolation scope's baggage. It always has one
-        return Scope.get_isolation_scope().get_baggage()
+        return self.get_isolation_scope().get_baggage()
 
     def get_trace_context(self):
         # type: () -> Any
@@ -583,12 +614,18 @@ class Scope(object):
     def iter_trace_propagation_headers(self, *args, **kwargs):
         # type: (Any, Any) -> Generator[Tuple[str, str], None, None]
         """
-        Return HTTP headers which allow propagation of trace data. Data taken
-        from the span representing the request, if available, or the current
-        span on the scope if not.
+        Return HTTP headers which allow propagation of trace data.
+
+        If a span is given, the trace data will taken from the span.
+        If no span is given, the trace data is taken from the scope.
         """
-        client = Scope.get_client()
+        client = self.get_client()
         if not client.options.get("propagate_traces"):
+            warnings.warn(
+                "The `propagate_traces` parameter is deprecated. Please use `trace_propagation_targets` instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
             return
 
         span = kwargs.pop("span", None)
@@ -605,13 +642,13 @@ class Scope(object):
                     yield header
             else:
                 # otherwise try headers from current scope
-                current_scope = Scope.get_current_scope()
+                current_scope = self.get_current_scope()
                 if current_scope._propagation_context is not None:
                     for header in current_scope.iter_headers():
                         yield header
                 else:
                     # otherwise fall back to headers from isolation scope
-                    isolation_scope = Scope.get_isolation_scope()
+                    isolation_scope = self.get_isolation_scope()
                     if isolation_scope._propagation_context is not None:
                         for header in isolation_scope.iter_headers():
                             yield header
@@ -621,11 +658,11 @@ class Scope(object):
         if self._propagation_context is not None:
             return self._propagation_context
 
-        current_scope = Scope.get_current_scope()
+        current_scope = self.get_current_scope()
         if current_scope._propagation_context is not None:
             return current_scope._propagation_context
 
-        isolation_scope = Scope.get_isolation_scope()
+        isolation_scope = self.get_isolation_scope()
         if isolation_scope._propagation_context is not None:
             return isolation_scope._propagation_context
 
@@ -656,6 +693,10 @@ class Scope(object):
 
         self._propagation_context = None
 
+        # self._last_event_id is only applicable to isolation scopes
+        self._last_event_id = None  # type: Optional[str]
+        self._flags = None  # type: Optional[FlagBuffer]
+
     @_attr_setter
     def level(self, value):
         # type: (LogLevelStr) -> None
@@ -754,7 +795,7 @@ class Scope(object):
         # type: (Optional[Dict[str, Any]]) -> None
         """Sets a user for the scope."""
         self._user = value
-        session = Scope.get_isolation_scope()._session
+        session = self.get_isolation_scope()._session
         if session is not None:
             session.update(user=value)
 
@@ -868,14 +909,17 @@ class Scope(object):
 
     def add_attachment(
         self,
-        bytes=None,  # type: Optional[bytes]
+        bytes=None,  # type: Union[None, bytes, Callable[[], bytes]]
         filename=None,  # type: Optional[str]
         path=None,  # type: Optional[str]
         content_type=None,  # type: Optional[str]
         add_to_transactions=False,  # type: bool
     ):
         # type: (...) -> None
-        """Adds an attachment to future events sent."""
+        """Adds an attachment to future events sent from this scope.
+
+        The parameters are the same as for the :py:class:`sentry_sdk.attachments.Attachment` constructor.
+        """
         self._attachments.append(
             Attachment(
                 bytes=bytes,
@@ -896,7 +940,7 @@ class Scope(object):
         :param hint: An optional value that can be used by `before_breadcrumb`
             to customize the breadcrumbs that are emitted.
         """
-        client = Scope.get_client()
+        client = self.get_client()
 
         if not client.is_active():
             logger.info("Dropped breadcrumb because no client bound")
@@ -935,7 +979,7 @@ class Scope(object):
         transaction=None,
         instrumenter=INSTRUMENTER.SENTRY,
         custom_sampling_context=None,
-        **kwargs
+        **kwargs,
     ):
         # type: (Optional[Transaction], str, Optional[SamplingContext], Unpack[TransactionKwargs]) -> Union[Transaction, NoOpSpan]
         """
@@ -962,7 +1006,8 @@ class Scope(object):
 
         :param transaction: The transaction to start. If omitted, we create and
             start a new transaction.
-        :param instrumenter: This parameter is meant for internal use only.
+        :param instrumenter: This parameter is meant for internal use only. It
+            will be removed in the next major version.
         :param custom_sampling_context: The transaction's custom sampling context.
         :param kwargs: Optional keyword arguments to be passed to the Transaction
             constructor. See :py:class:`sentry_sdk.tracing.Transaction` for
@@ -970,13 +1015,15 @@ class Scope(object):
         """
         kwargs.setdefault("scope", self)
 
-        client = Scope.get_client()
+        client = self.get_client()
 
         configuration_instrumenter = client.options["instrumenter"]
 
         if instrumenter != configuration_instrumenter:
             return NoOpSpan()
 
+        try_autostart_continuous_profiler()
+
         custom_sampling_context = custom_sampling_context or {}
 
         # kwargs at this point has type TransactionKwargs, since we have removed
@@ -996,12 +1043,16 @@ class Scope(object):
         sampling_context.update(custom_sampling_context)
         transaction._set_initial_sampling_decision(sampling_context=sampling_context)
 
-        profile = Profile(transaction)
-        profile._set_initial_sampling_decision(sampling_context=sampling_context)
-
-        # we don't bother to keep spans if we already know we're not going to
-        # send the transaction
         if transaction.sampled:
+            profile = Profile(
+                transaction.sampled, transaction._start_timestamp_monotonic_ns
+            )
+            profile._set_initial_sampling_decision(sampling_context=sampling_context)
+
+            transaction._profile = profile
+
+            # we don't bother to keep spans if we already know we're not going to
+            # send the transaction
             max_spans = (client.options["_experiments"].get("max_spans")) or 1000
             transaction.init_span_recorder(maxlen=max_spans)
 
@@ -1023,11 +1074,22 @@ class Scope(object):
         one is not already in progress.
 
         For supported `**kwargs` see :py:class:`sentry_sdk.tracing.Span`.
+
+        The instrumenter parameter is deprecated for user code, and it will
+        be removed in the next major version. Going forward, it should only
+        be used by the SDK itself.
         """
+        if kwargs.get("description") is not None:
+            warnings.warn(
+                "The `description` parameter is deprecated. Please use `name` instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
         with new_scope():
             kwargs.setdefault("scope", self)
 
-            client = Scope.get_client()
+            client = self.get_client()
 
             configuration_instrumenter = client.options["instrumenter"]
 
@@ -1035,7 +1097,7 @@ class Scope(object):
                 return NoOpSpan()
 
             # get current span or transaction
-            span = self.span or Scope.get_isolation_scope().span
+            span = self.span or self.get_isolation_scope().span
 
             if span is None:
                 # New spans get the `trace_id` from the scope
@@ -1051,8 +1113,10 @@ class Scope(object):
 
             return span
 
-    def continue_trace(self, environ_or_headers, op=None, name=None, source=None):
-        # type: (Dict[str, Any], Optional[str], Optional[str], Optional[str]) -> Transaction
+    def continue_trace(
+        self, environ_or_headers, op=None, name=None, source=None, origin="manual"
+    ):
+        # type: (Dict[str, Any], Optional[str], Optional[str], Optional[str], str) -> Transaction
         """
         Sets the propagation context from environment or headers and returns a transaction.
         """
@@ -1061,6 +1125,7 @@ class Scope(object):
         transaction = Transaction.continue_from_headers(
             normalize_incoming_data(environ_or_headers),
             op=op,
+            origin=origin,
             name=name,
             source=source,
         )
@@ -1087,9 +1152,17 @@ class Scope(object):
 
         :returns: An `event_id` if the SDK decided to send the event (see :py:meth:`sentry_sdk.client._Client.capture_event`).
         """
+        if disable_capture_event.get(False):
+            return None
+
         scope = self._merge_scopes(scope, scope_kwargs)
 
-        return Scope.get_client().capture_event(event=event, hint=hint, scope=scope)
+        event_id = self.get_client().capture_event(event=event, hint=hint, scope=scope)
+
+        if event_id is not None and event.get("type") != "transaction":
+            self.get_isolation_scope()._last_event_id = event_id
+
+        return event_id
 
     def capture_message(self, message, level=None, scope=None, **scope_kwargs):
         # type: (str, Optional[LogLevelStr], Optional[Scope], Any) -> Optional[str]
@@ -1109,6 +1182,9 @@ class Scope(object):
 
         :returns: An `event_id` if the SDK decided to send the event (see :py:meth:`sentry_sdk.client._Client.capture_event`).
         """
+        if disable_capture_event.get(False):
+            return None
+
         if level is None:
             level = "info"
 
@@ -1134,34 +1210,25 @@ class Scope(object):
 
         :returns: An `event_id` if the SDK decided to send the event (see :py:meth:`sentry_sdk.client._Client.capture_event`).
         """
+        if disable_capture_event.get(False):
+            return None
+
         if error is not None:
             exc_info = exc_info_from_error(error)
         else:
             exc_info = sys.exc_info()
 
         event, hint = event_from_exception(
-            exc_info, client_options=Scope.get_client().options
+            exc_info, client_options=self.get_client().options
         )
 
         try:
             return self.capture_event(event, hint=hint, scope=scope, **scope_kwargs)
         except Exception:
-            self._capture_internal_exception(sys.exc_info())
+            capture_internal_exception(sys.exc_info())
 
         return None
 
-    def _capture_internal_exception(
-        self, exc_info  # type: Any
-    ):
-        # type: (...) -> Any
-        """
-        Capture an exception that is likely caused by a bug in the SDK
-        itself.
-
-        These exceptions do not end up in Sentry and are just logged instead.
-        """
-        logger.error("Internal error in sentry_sdk", exc_info=exc_info)
-
     def start_session(self, *args, **kwargs):
         # type: (*Any, **Any) -> None
         """Starts a new session."""
@@ -1169,7 +1236,7 @@ class Scope(object):
 
         self.end_session()
 
-        client = Scope.get_client()
+        client = self.get_client()
         self._session = Session(
             release=client.options.get("release"),
             environment=client.options.get("environment"),
@@ -1185,7 +1252,7 @@ class Scope(object):
 
         if session is not None:
             session.close()
-            Scope.get_client().capture_session(session)
+            self.get_client().capture_session(session)
 
     def stop_auto_session_tracking(self, *args, **kwargs):
         # type: (*Any, **Any) -> None
@@ -1261,6 +1328,17 @@ class Scope(object):
             self._breadcrumbs
         )
 
+        # Attempt to sort timestamps
+        try:
+            for crumb in event["breadcrumbs"]["values"]:
+                if isinstance(crumb["timestamp"], str):
+                    crumb["timestamp"] = datetime_from_isoformat(crumb["timestamp"])
+
+            event["breadcrumbs"]["values"].sort(key=lambda crumb: crumb["timestamp"])
+        except Exception as err:
+            logger.debug("Error when sorting breadcrumbs", exc_info=err)
+            pass
+
     def _apply_user_to_event(self, event, hint, options):
         # type: (Event, Hint, Optional[Dict[str, Any]]) -> None
         if event.get("user") is None and self._user is not None:
@@ -1305,6 +1383,14 @@ class Scope(object):
             else:
                 contexts["trace"] = self.get_trace_context()
 
+    def _apply_flags_to_event(self, event, hint, options):
+        # type: (Event, Hint, Optional[Dict[str, Any]]) -> None
+        flags = self.flags.get()
+        if len(flags) > 0:
+            event.setdefault("contexts", {}).setdefault("flags", {}).update(
+                {"values": flags}
+            )
+
     def _drop(self, cause, ty):
         # type: (Any, str) -> Optional[Any]
         logger.info("%s (%s) dropped event", ty, cause)
@@ -1318,9 +1404,9 @@ class Scope(object):
         exc_info = hint.get("exc_info")
         if exc_info is not None:
             error_processors = chain(
-                Scope.get_global_scope()._error_processors,
-                Scope.get_isolation_scope()._error_processors,
-                Scope.get_current_scope()._error_processors,
+                self.get_global_scope()._error_processors,
+                self.get_isolation_scope()._error_processors,
+                self.get_current_scope()._error_processors,
             )
 
             for error_processor in error_processors:
@@ -1403,6 +1489,7 @@ class Scope(object):
 
         if not is_transaction and not is_check_in:
             self._apply_breadcrumbs_to_event(event, hint, options)
+            self._apply_flags_to_event(event, hint, options)
 
         event = self.run_error_processors(event, hint)
         if event is None:
@@ -1445,6 +1532,12 @@ class Scope(object):
             self._propagation_context = scope._propagation_context
         if scope._session:
             self._session = scope._session
+        if scope._flags:
+            if not self._flags:
+                self._flags = deepcopy(scope._flags)
+            else:
+                for flag in scope._flags.get():
+                    self._flags.set(flag["flag"], flag["result"])
 
     def update_from_kwargs(
         self,
@@ -1479,6 +1572,17 @@ class Scope(object):
             self._type,
         )
 
+    @property
+    def flags(self):
+        # type: () -> FlagBuffer
+        if self._flags is None:
+            max_flags = (
+                self.get_client().options["_experiments"].get("max_flags")
+                or DEFAULT_FLAG_CAPACITY
+            )
+            self._flags = FlagBuffer(capacity=max_flags)
+        return self._flags
+
 
 @contextmanager
 def new_scope():
@@ -1640,3 +1744,6 @@ def should_send_default_pii():
 
 # Circular imports
 from sentry_sdk.client import NonRecordingClient
+
+if TYPE_CHECKING:
+    import sentry_sdk.client
